import socket
from dataclasses import dataclass, field
from typing import List, Optional

from mag_tools.bean.sys.cpu import Cpu
from mag_tools.bean.sys.disk import Disk
from mag_tools.bean.sys.disk_partition import DiskPartition
from mag_tools.bean.sys.memory import Memory
from mag_tools.bean.sys.mother_board import Motherboard
from mag_tools.model.computer_type import ComputerType


@dataclass
class Computer:
    """
    计算机类
    """
    type: ComputerType
    id: Optional[str] = None
    name: Optional[str] = None
    cpu: Optional[Cpu] = None
    memory: Optional[Memory] = None
    disks: List[Disk] = field(default_factory=list)
    mother_board: Optional[Motherboard] = None
    description: Optional[str] = None

    @classmethod
    def get_info(cls):
        """
        获取当前系统的CPU、内存和磁盘信息，并返回一个Computer实例
        """
        # 获取CPU信息
        cpu = Cpu.get_info()

        # 获取内存信息
        memory = Memory.get_info()

        # 获取磁盘信息
        disks = Disk.get_info()

        #获取主板信息
        mother_board = Motherboard.get_info()

        id_ = cls.__make_id(cpu, mother_board)
        cpu.computer_id = id_
        memory.computer_id = id_
        mother_board.computer_id = id_

        for disk in disks:
            disk.computer_id = id_

        for module in memory.modules:
            module.computer_id = id_

        computer = Computer(
            id=id_,
            type=ComputerType.DESKTOP,
            name=socket.gethostname(),
            cpu=cpu,
            memory=memory,
            disks=disks,
            mother_board=mother_board
        )

        return computer

    @classmethod
    def __make_id(cls, cpu: Optional[Cpu], mother_board: Optional[Motherboard]) -> str:
        """
        返回计算机对象的ID
        """
        cpu_serial = cpu.serial_number if cpu else ""
        motherboard_serial = mother_board.serial_number if mother_board else ""

        combined_serials = f"{cpu_serial}{motherboard_serial}"
        return str(hash(combined_serials))

    def __str__(self):
        """
        返回计算机参数的字符串表示
        """
        parts = [f"Computer(type='{self.type}'"]
        for attr, value in self.__dict__.items():
            if value is not None:
                parts.append(f"{attr}='{value}'")
        parts.append(")")
        return ", ".join(parts)

    def __hash__(self):
        """
        返回计算机对象的哈希值
        """
        cpu_serial = self.cpu.serial_number if self.cpu else ""
        motherboard_serial = self.mother_board.serial_number if self.mother_board else ""
        memory_serials = "".join([module.serial_number for module in self.memory.modules]) if self.memory else ""
        disk_serials = "".join([disk.serial_number for disk in self.disks])

        combined_serials = f"{cpu_serial}{motherboard_serial}{memory_serials}{disk_serials}"
        return hash(combined_serials)


import inspect
from dataclasses import dataclass, field
from typing import Any, Dict, Optional, List

from mag_tools.bean.data_format import DataFormat
from mag_tools.bean.text_format import TextFormat
from mag_tools.model.justify_type import JustifyType
from mag_tools.utils.data.string_format import StringFormat


@dataclass
class BaseData:
    _text_format: TextFormat = field(default_factory=lambda: TextFormat(
        number_per_line=4,
        justify_type=JustifyType.LEFT,
        at_header='',
        decimal_places=4,
        decimal_places_of_zero=1
    ))
    _data_formats: Dict[str, DataFormat] = field(default_factory=dict, metadata={"description": "数据格式"})

    def set_pad_lengths(self, pad_lengths: dict[str, int]):
        for arg_name, value in pad_lengths.items():
            data_format = self.get_data_format(arg_name)
            if data_format is None:
                data_format = self._text_format.get_data_format_by_value(vars(self).get(arg_name))
                if data_format:
                    self._data_formats[arg_name] = data_format
            if data_format:
                data_format.pad_length = value

    def set_same_pad_length(self, pad_length:int):
        for data_format in self._data_formats.values():
            data_format.pad_length = pad_length

    def set_justify_type(self, justify_type: JustifyType):
        self._text_format.justify_type = justify_type
        for data_format in self._data_formats.values():
            data_format.justify_type = justify_type

    def set_decimal_places(self, decimal_places):
        self._text_format.decimal_places = decimal_places
        for data_format in self._data_formats.values():
            data_format.decimal_places = decimal_places

    def set_decimal_places_of_zero(self, decimal_places_of_zero):
        self._text_format.decimal_places_of_zero = decimal_places_of_zero
        for data_format in self._data_formats.values():
            data_format.decimal_places_of_zero = decimal_places_of_zero

    def set_number_per_line(self, number_per_line):
        self._text_format.number_per_line = number_per_line

    def set_at_header(self, at_header):
        self._text_format.at_header = at_header

    def set_scientific(self, scientific):
        self._text_format.scientific = scientific

    def get_text(self, arg_names: List[str], delimiter: Optional[str] = None) -> str:
        """
        根据参数名数组拼成一个字符串。
        :param arg_names: 类成员变量的名字数组
        :param delimiter: 分隔符
        :return: 拼接后的字符串
        """

        if delimiter is None:
            delimiter = ''

        strings = []
        need_space = False
        for arg_name in arg_names:
            data_format = self.get_data_format(arg_name)
            value_str = str(vars(self).get(arg_name))
            pad_length = max(len(value_str), data_format.pad_length or 0)
            need_space = need_space or pad_length == len(value_str)

            strings.append(StringFormat.pad_value(value_str, pad_length, data_format.justify_type))

        text = (' ' if need_space else '').join(strings)
        if delimiter:
            text = text.rstrip(delimiter)  # 删除末尾的分隔符
        return text

    def get_data_format(self, arg_name: str) -> DataFormat:
        if arg_name not in self._data_formats:
            for name, value in vars(self).items():
                if name not in ['_text_format', '_data_formats']:
                    data_format = self._text_format.get_data_format_by_value(value)
                    if data_format:
                        self._data_formats[name] = data_format

        return self._data_formats.get(arg_name)

    @property
    def variables_map(self) -> Dict[str, Any]:
        return {
            k: repr(v) for k, v in self.__dict__.items()
            if not (k.startswith('__') and k.endswith('__')) and k not in {'_text_format', '_data_formats'}
        }

class TestData(BaseData):
    def __init__(self, name: Optional[str] = None, age: Optional[int] = None, height: Optional[float] = None):
        super().__init__()

        self.name = name
        self.age = age
        self.height = height


if __name__ == '__main__':
    data = TestData('xlcao', 12, 1)
    # print(data.get_text(['name', 'age', 'height'], ','))
    #
    # data.set_pad_lengths({'name': 20, 'age': 8, 'height': 10})
    # print(data.get_text(['name', 'age', 'height'], ','))
    #
    # data.set_justify_type(JustifyType.LEFT)
    # print(data.get_text(['name', 'age', 'height'], ','))
    #
    # data.set_justify_type(JustifyType.CENTER)
    # print(data.get_text(['name', 'age', 'height'], ','))

    map_ = data.variables_map
    for k_, v_ in map_.items():
        print(f'{k_}: {v_}')


from dataclasses import dataclass
from typing import Optional
from mag_tools.model.justify_type import JustifyType

@dataclass
class DataFormat:
    justify_type: JustifyType = JustifyType.LEFT
    decimal_places: int = 2
    decimal_places_of_zero: int = 1
    pad_length: Optional[int] = None
    scientific: bool = False

    def __str__(self):
        """
        返回 DataFormat 实例的字符串表示。
        :return: DataFormat 实例的字符串表示。
        """
        return f"DataFormat({', '.join(f'{key}={value}' for key, value in vars(self).items())})"


import json
from typing import Any, Dict, List, Tuple

from mag_tools.utils.data.string_utils import StringUtils


class MapUtils:
    @staticmethod
    def to_property(obj: Dict[str, str]):
        return '\n'.join(MapUtils.keypair_to_string((k, v)) for k, v in obj.items())

    @staticmethod
    def keypair_to_string(keypair: Tuple[str, str]) -> str:
        return f'{keypair[0]} = {keypair[1]}'

    @staticmethod
    def keys_to_underline(obj: [Dict, List, Tuple]):
        """
        将对象中的键值转换为下划线格式
        """
        if isinstance(obj, dict):
            new_map = {}
            for k, v in obj.items():
                new_map[StringUtils.hump2underline(k)] = MapUtils.keys_to_underline(v)
            obj = new_map
        elif isinstance(obj, list):
            obj = [MapUtils.keys_to_underline(i) for i in obj]
        elif isinstance(obj, tuple):
            obj = MapUtils.keys_to_underline(obj)

        return obj

    @staticmethod
    def keys_to_hump(obj: [Dict, List, Tuple]):
        """
        将对象中的键值转换为驼峰格式
        """
        if isinstance(obj, dict):
            new_map = {}
            for k, v in obj.items():
                new_map[StringUtils.underline2hump(k)] = MapUtils.keys_to_hump(v)
            obj = new_map
        elif isinstance(obj, list):
            obj = [MapUtils.keys_to_hump(i) for i in obj]
        elif isinstance(obj, tuple):
            obj = MapUtils.keys_to_hump(obj)

        return obj

if __name__ == '__main__':
    # 示例使用
    _s = '{"code":200,"message":"OK","timestamp":"2025-02-13T18:25:39.843341498","totalCount":1,"data":[{"userSn":1,"secret":null,"hashCount":162706,"salt":"kWmg_tX*JWrL0EK-","makeTime":"2024-07-26T16:03:19","errorCount":0,"nextChangeTime":"2024-10-24T16:03:19"}],"success":true}'
    _map = json.loads(_s)
    print(MapUtils.keys_to_underline(_map))


import datetime
from typing import Optional


class CheckNode:
    def __init__(self, sn: int, name: Optional[str], time: datetime, last_cost: int, total_cost: int):
        """
        初始化 CheckNode 类的实例
        :param sn: 节点序号
        :param name: 节点名称
        :param time: 节点时间
        :param last_cost: 上次到当前节点的时间差，单位：毫秒
        :param total_cost: 从开始到当前节点的总时间差，单位：毫秒
        """
        self.__sn = sn
        self.name = name if name else f"节点{sn}"
        self.__time = time
        self.__last_cost = last_cost
        self.__total_cost = total_cost

    def __str__(self):
        """
        返回节点的字符串表示
        :return: 节点的字符串表示
        """
        return f"sn={self.__sn};\nname={self.name};\ntime={self.__time};\nlast_cost={self.__last_cost};\ntotal_cost={self.__total_cost};\n"

    def log(self):
        """
        返回节点的日志信息
        :return: 节点的日志信息
        """
        total = f"{self.__total_cost / 1000}毫秒" if self.__total_cost > 1000 else f"{self.__total_cost}纳秒"
        last = f"{self.__last_cost / 1000}毫秒" if self.__last_cost > 1000 else f"{self.__last_cost}纳秒"

        return f"  节点号：{self.__sn};\n  节点名：{self.name};\n  执行时间：{self.__time};\n  本节点花费时间：{last};\n  总花费时间：{total};\n"



from mag_tools.model.justify_type import JustifyType

from mag_tools.bean.data_format import DataFormat
from mag_tools.utils.data.string_format import StringFormat
import unittest


class TestStringFormat(unittest.TestCase):
    def test_pad_string(self):
        # 测试右对齐
        self.assertEqual(StringFormat.pad_value("test", 10), "      test")
        # 测试居中对齐
        self.assertEqual(StringFormat.pad_value("test", 10, JustifyType.CENTER), "   test   ")
        # 测试左对齐
        self.assertEqual(StringFormat.pad_value("test", 10, JustifyType.LEFT), "test      ")
        # 测试字符串长度大于目标长度
        self.assertEqual(StringFormat.pad_value("test", 2), "test")

    def test_pad_strings(self):
        # 测试多个字符串右对齐
        strings = ["test1", "test2", "test3"]
        expected_output = "     test1     test2     test3"
        self.assertEqual(expected_output, StringFormat.pad_values(strings, 10))
        # 测试多个字符串居中对齐
        expected_output = " test1   test2   test3  "
        self.assertEqual(expected_output, StringFormat.pad_values(strings, 8, JustifyType.CENTER))
        # 测试多个字符串左对齐
        expected_output = "test1 test2 test3"
        self.assertEqual(expected_output, StringFormat.pad_values(strings, 5, JustifyType.LEFT))

    def test_format_number(self):
        # 测试整数格式化
        data_format = DataFormat()
        self.assertEqual("123", StringFormat.format_number(123, data_format))

        # 测试浮点数格式化
        data_format = DataFormat(decimal_places_of_zero=2)
        self.assertEqual("123.46", StringFormat.format_number(123.456, data_format))
        self.assertEqual("123.00", StringFormat.format_number(123.0, data_format))

        # 测试科学计数法格式化
        data_format = DataFormat(decimal_places=5, scientific=True)
        self.assertEqual(StringFormat.format_number(1e9, data_format), "1.00000e9")
        self.assertEqual(StringFormat.format_number(1e-4, data_format), "1.00000e-4")

        # 测试非数值类型
        self.assertEqual(StringFormat.format_number("text", data_format), "text")

    def test_format(self):
        print(StringFormat.format("{random_text(5)}"))
        print(StringFormat.format("{random_string(3)}"))
        print(StringFormat.format("{random_chinese(12)}"))
        print(StringFormat.format("{random_number(4)}"))
        print(StringFormat.format("{random_date(2024,%Y-%m-%d)}"))
        print(StringFormat.format("{today(%Y%m%d)}"))
        print(StringFormat.format("{current(%Y-%m-%d %H:%M:%S)}"))
        print(StringFormat.format("{random_int(20, 30)}"))


if __name__ == '__main__':
    unittest.main()



import unittest

from mag_tools.bean.data_format import DataFormat

from mag_tools.model.data_type import DataType

from mag_tools.utils.data.string_utils import StringUtils
from mag_tools.utils.data.string_format import StringFormat


class TestStringStringUtils(unittest.TestCase):

    def test_get_before_keyword(self):
        self.assertEqual(StringUtils.pick_head("hello world", "world"), "hello ")
        self.assertEqual(StringUtils.pick_head("hello world", " "), "hello")
        self.assertEqual(StringUtils.pick_head("hello world", "hello"), "")

    def test_get_after_keyword(self):
        self.assertEqual(StringUtils.pick_tail("hello world", "hello"), " world")
        self.assertIsNone(StringUtils.pick_tail("hello world", "world"))
        self.assertIsNone(StringUtils.pick_tail("hello world", "notfound"))

    def test_split_by_keyword(self):
        _input_strings = [
            "{打开工区}",
            "打开工区",
            "GridControl{T}",
            "打开工区(AAA)",
            "打开工区[AAA]BBB",
            "打开工区'AAA'BBB",
            '打开工区"AAA"BBB',
        ]

        _keywords = ['{}', '{}', '()', '[]', "''", '""']

        for _input_string, _keyword in zip(_input_strings, _keywords):
            _name, _function, _remainder = StringUtils.split_by_keyword(_input_string, _keyword)
            print(f"Input: {_input_string} -> Name: {_name}, Function: {_function}, Remainder: {_remainder}")

    def test_split_name_id(self):
        self.assertEqual(StringUtils.split_name_id("名称(标识)"), ("名称", "标识"))
        self.assertEqual(StringUtils.split_name_id("名称（标识）"), ("名称", "标识"))
        self.assertEqual(StringUtils.split_name_id("名称标识"), ("名称标识", None))

    def test_parse_function(self):
        self.assertEqual(StringUtils.parse_function("test(arg1, arg2)"), ("test", ["arg1", "arg2"]))
        self.assertEqual(StringUtils.parse_function("test()"), ("test", []))
        with self.assertRaises(ValueError):
            StringUtils.parse_function("test")

    def test_to_chinese_number(self):
        self.assertEqual(StringUtils.to_chinese_number(0), "零")
        self.assertEqual(StringUtils.to_chinese_number(10), "十")
        self.assertEqual(StringUtils.to_chinese_number(110), "一百一十")
        self.assertEqual(StringUtils.to_chinese_number(1234), "一千二百三十四")
        self.assertEqual(StringUtils.to_chinese_number(10001), "一万零一")

    def test_to_value(self):
        # 测试整数转换
        self.assertEqual(StringUtils.to_value("123", int), 123)
        # 测试浮点数转换
        self.assertEqual(StringUtils.to_value("123.45", float), 123.45)
        # 测试布尔值转换
        self.assertTrue(StringUtils.to_value("true", bool))
        self.assertFalse(StringUtils.to_value("false", bool))
        # 测试列表转换
        self.assertEqual(StringUtils.to_value("[1, 2, 3]", list), [1, 2, 3])
        # 测试字典转换
        self.assertEqual(StringUtils.to_value("{'key': 'value'}", dict), {'key': 'value'})
        # 测试默认类型转换
        self.assertEqual(StringUtils.to_value("text"), "text")

    def test_format_value(self):
        # 测试浮点数格式化
        data_format = DataFormat(decimal_places_of_zero=2)
        self.assertEqual(StringFormat.format_number(123.456, data_format), "123.46")
        self.assertEqual(StringFormat.format_number(123.0, data_format), "123.00")
        # 测试科学计数法格式化
        self.assertEqual(StringFormat.format_number(1e9, DataFormat(decimal_places=6, scientific=True)), "1.000000e9")
        # 测试整数格式化
        self.assertEqual(StringFormat.format_number(123, DataFormat()), "123")

    def test_float_to_scientific(self):
        # 测试默认小数位数
        self.assertEqual(StringUtils.float_to_scientific(1000000000.0), "1.000000e9")
        self.assertEqual(StringUtils.float_to_scientific(0.000123), "1.230000e-4")

        # 测试指定小数位数
        self.assertEqual(StringUtils.float_to_scientific(1000000000.0, 5), "1.00000e9")
        self.assertEqual(StringUtils.float_to_scientific(0.000123, 3), "1.230e-4")

        # 测试负数
        self.assertEqual(StringUtils.float_to_scientific(-1000000000.0), "-1.000000e9")
        self.assertEqual(StringUtils.float_to_scientific(-0.000123), "-1.230000e-4")

        # 测试小数位数不足时补零
        self.assertEqual(StringUtils.float_to_scientific(1.23, 5), "1.23000e0")
        self.assertEqual(StringUtils.float_to_scientific(1.0, 5), "1.00000e0")

    def test_parse_strings_to_map(self):
        # 测试用例 1：使用空格作为分隔符
        strs = [
            "key1 value1",
            "key2 value2",
            "key3 value3"
        ]
        expected_output = {
            "key1": "value1",
            "key2": "value2",
            "key3": "value3"
        }
        result = StringUtils.parse_strings_to_map(strs)
        self.assertEqual(result, expected_output)

        # 测试用例 2：使用逗号作为分隔符
        strs = [
            "key1,value1",
            "key2,value2",
            "key3,value3"
        ]
        expected_output = {
            "key1": "value1",
            "key2": "value2",
            "key3": "value3"
        }
        result = StringUtils.parse_strings_to_map(strs, delimiter=',')
        self.assertEqual(result, expected_output)

        # 测试用例 3：字符串中没有分隔符
        strs = [
            "key1 value1",
            "key2value2",  # 没有空格
            "key3 value3"
        ]
        with self.assertRaises(ValueError):
            StringUtils.parse_strings_to_map(strs)

        # 测试用例 4：字符串中有多个分隔符
        strs = [
            "key1 value1",
            "key2  value2",  # 多个空格
            "key3 value3"
        ]
        expected_output = {
            "key1": "value1",
            "key2": " value2",
            "key3": "value3"
        }
        result = StringUtils.parse_strings_to_map(strs)
        self.assertEqual(result, expected_output)

    def test_underline2hump(self):
        self.assertEqual(StringUtils.underline2hump('family_address'), 'familyAddress')
        self.assertEqual(StringUtils.underline2hump('my_variable_name'), 'myVariableName')
        self.assertEqual(StringUtils.underline2hump('test_case'), 'testCase')
        self.assertEqual(StringUtils.underline2hump('example_string'), 'exampleString')

    def test_hump2underline(self):
        self.assertEqual(StringUtils.hump2underline('familyAddress'), 'family_address')
        self.assertEqual(StringUtils.hump2underline('myVariableName'), 'my_variable_name')
        self.assertEqual(StringUtils.hump2underline('testCase'), 'test_case')
        self.assertEqual(StringUtils.hump2underline('exampleString'), 'example_string')

if __name__ == '__main__':
    unittest.main()
